// C++ Sample file to test ActiveX Server Object CornerDetector.CoCornerDetector

#include <stdio.h>
#include <conio.h>
//REQUIRED FOR COM EXTENDED FUNCTIONS
#define _WIN32_WINNT 0x0400
//LOAD TYPE INFORMATION DATA
#import "CornerDetector.tlb" no_namespace named_guids

//THIS PROJECT REQUIRES MICROSOFT VISION SDK. 
//PLEASE MAKE SURE TO HAVE IT INSTALLED ON YOUR MACHINE
#include "viscore.h"

main()
{
	long max_corners = 100;
	float *coords = (float*)malloc(2*max_corners*sizeof(float));
	float *eig_image;


	//INITIALIZE COM
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if(FAILED(hr))
	{
		printf("Unable to Initialize COM\n");
		return -1;
	}

	//CREATE ACTIVEX OBJECT
	ICornerDetector *ptr;
	hr = CoCreateInstance(	CLSID_CoCornerDetector, NULL, CLSCTX_INPROC_SERVER, 
							IID_ICornerDetector, (void**)&ptr);	
	if(FAILED(hr))
	{
		printf("Unable to Create Object\n");
		return -1;
	}

	CVisGrayByteImage gray;
	printf("Reading gray image ...\n");
	if (!gray.FReadFile("test.bmp"))
	{
		printf("Could not open image\n");
		return -1;
	}

	//CONFIGURE OBJECT COMPATIBLE WITH IMAGE AND CORNER ARRAY SIZES
	long image_lines = gray.Height();
	long image_columns = gray.Width();	
	hr = ptr->put_lines(image_lines);
	hr = ptr->put_columns(image_columns);
	hr = ptr->put_max_corners(max_corners);

	//ALLOCATE THE CORNER DETECTOR
	hr = ptr->create_corner_detector();
	if(FAILED(hr))
	{
		printf("Unable to Allocate Internal Object Resources\n");
		return -1;
	}

	//ASK FOR THE DIRECT ACCESS INTERFACE
	_ICornerDetector *ptr_direct;
	hr = ptr->QueryInterface(IID__ICornerDetector, (void**)&ptr_direct);
	if(FAILED(hr))
	{
		printf("Unable to access direct interface\n");
		return -1;
	}

	//DETECT THE CORNERS
	long corners_detected = max_corners;
	hr = ptr_direct->_detect_corners(gray.MemBlock().PbData(), coords, &corners_detected );
	if(FAILED(hr))
	{
		printf("Corner detection function failed\n");
		return -1;
	}

	
	// convert gray image to color image for visualization purposes	
	CVisRGBAByteImage color;
	color.Allocate(gray.Rect());
	gray.CopyPixelsTo(color);

	//Plot detected corners on the image
	HDC hdcImage;	
	hdcImage = color.Hdc();
	if (hdcImage != 0)
	{
		HPEN pen = ::CreatePen(PS_SOLID, 2, RGB(255,0,0));
		::SelectObject(hdcImage, pen );
		float x,y;
		for(int i = 0; i < corners_detected; i++ )
		{
			x = coords[2*i];
			y = coords[2*i+1];
			::Ellipse(hdcImage, x-2, y-2, x+2, y+2);
		}
		color.DestroyHdc();
	}
	else
		printf("Could not create device context for test image\n");


	printf("Displaying detected corners image\n");
	CVisImageWnd wcolor(color);	
	printf("Press any key to continue...\n");
	_getch();

	printf("Writing image with detected corners...\n");
	if(!color.FWriteFile("corners.bmp"))
	{
		printf("Could not save image\n");
		return -1;
	}

	//GET THE EIGENIMAGE
	hr = ptr_direct->_get_eig_image(&eig_image); //direct access to internal pointer
												 //do not write on this memory

	if(FAILED(hr))
	{
		printf("Unable to access eigenimage\n");
		return -1;
	}
	
	//CONVERT TO COLOR AND NORMALIZE BRIGHTNESS FOR DISPLAY
	CVisFloatImage eigfloat(image_columns, image_lines, 1, evisimoptDefault, (byte*)eig_image);
	CVisRGBAByteImage eigcolor;
	eigcolor.Allocate(eigfloat.Rect());
	eigfloat.CopyPixelsTo(eigcolor, evisnormalizeRelative);


	//DISPLAY EIGENIMAGE
	printf("Displaying image of minimum eigenvalues\n");
	CVisImageWnd weig(eigcolor);
	printf("Press any key to continue...\n");
	_getch();

	//CLOSE EVERYTHING
	ptr->destroy_corner_detector();
	ptr_direct->Release();
	ptr->Release();
	free(coords);

	CoUninitialize();
	
	printf("Done\n");
	return 0;
}